import argparse
import logging
import pathlib
import sys
import time
import getpass
import json
import numpy as np
import time
from influxdb import InfluxDBClient


try:
    import labRemote
except:
    print("Couldn't find the labRemote Python libraries. Make sure to add them to your $PYTHONPATH first by either doing")
    print()
    print("     export PYTHONPATH=/path/to/labRemote/build/lib:$PYTHONPATH")
    print()
    print("     or writing \"/path/to/labRemote/build/lib\" in config.json (\"dcs_control\" section)")
    print()
    print("And make sure also that you have previously built labRemote enabling the Python bindings (cmake3 -DUSE_PYTHON=on)")
    exit(1)


def meas_voltage(hw, ps, args):
    return ps.measureVoltage()


def power_on(hw, ps, args):
    if args.voltage_level and args.current_protect:
        ps.setVoltageLevel(args.voltage_level)
        ps.setCurrentProtect(args.current_protect)
    ps.turnOn()
    return 0


def power_off(hw, ps, args):
    ps.turnOff()
    return 0


def setUpPassiveChannels(config, curve, what):
    
    if what == "turnOn":
        for elem in config[curve]["passive_channels"]:

            ps = hw.getPowerSupplyChannel(elem["name"])

            print()
            print("setting I to %2g A and V to %2g V on channel %s" % (elem["current"], elem["voltage"], elem["name"]))
            ps.setVoltageLevel(elem["voltage"])
            ps.setCurrentLevel(elem["current"])

            ps.turnOn()

    elif what == "turnOff":
        for elem in config[curve]["passive_channels"]:
            ps = hw.getPowerSupplyChannel(elem["name"])
            ps.turnOn()



def doSensorIV(config):

    ch = config["sensor_iv"]["target_channel"]
    ps = hw.getPowerSupplyChannel(ch["name"])

    print("Starting the sensor IV scan...")

    ps.setCurrentLevel(ch["setting_current"])
    ps.setCurrentProtect(ch["current_protection"])

    print("\ntime\t"+"voltageLevel\t"+"voltageValue\t"+"voltageStd\t"+"currentValue\t"+"currentStd")

    start = time.time()

    v = ch["initial_voltage"]

    vi = []
    while v <= ch["final_voltage"]:

        # turn everything on
        ps.turnOn()
        setUpPassiveChannels(config, "sensor_iv", "turnOn")

        ps.setVoltageLevel(v)

        vListPerStep = []
        iListPerStep = []

        for j in range(ch["measurements_per_step"]):
            time.sleep(ch["sleeping_time_s"])
            vListPerStep.append(ps.measureVoltage())
            iListPerStep.append(ps.measureCurrent())

        elapsedTime = time.time() - start

        vMean = np.mean(vListPerStep)
        vStd  = np.std(vListPerStep)
        iMean = np.mean(iListPerStep)
        iStd  = np.std(iListPerStep)

        vi.append([vMean, iMean])
        print("%.2g\t%.2g\t \t%.2g\t \t%.2g\t \t%.2g\t \t%.2g" % (elapsedTime, v, vMean, vStd, iMean, iStd))

        v += ch["step_size"]

        # turn everything off
        ps.turnOff()
        setUpPassiveChannels(config, "sensor_iv", "turnOff")

        if len(vi) > 1 and (abs(vi[-1][1] - vi[-2][1]) <= 0.01 and abs(vi[-1][1] - ch["current_protection"]) <= 0.01):
            break



def doModuleVI(config):


    ch = config["module_vi"]["target_channel"]
    ps = hw.getPowerSupplyChannel(ch["name"])

    print("Starting the module VI scan...")

    ps.setVoltageLevel(ch["setting_voltage"])
    ps.setVoltageProtect(ch["voltage_protection"])

    print("\ntime\t"+"CurrentLevel\t"+"voltageValue\t"+"voltageStd\t"+"currentValue\t"+"currentStd")

    start = time.time()

    i = ch["initial_current"]

    while i <= ch["final_current"]:

        # turn everything on
        ps.turnOn()
        setUpPassiveChannels(config, "sensor_iv", "turnOn")

        ps.setCurrentLevel(i)

        vListPerStep = []
        iListPerStep = []

        for j in range(ch["measurements_per_step"]):
            time.sleep(ch["sleeping_time_s"])
            vListPerStep.append(ps.measureVoltage())
            iListPerStep.append(ps.measureCurrent())

        elapsedTime = time.time() - start

        vMean = np.mean(vListPerStep)
        vStd  = np.std(vListPerStep)
        iMean = np.mean(iListPerStep)
        iStd  = np.std(iListPerStep)

        # turn everything off
        ps.turnOff()
        setUpPassiveChannels(config, "sensor_iv", "turnOff")

        print("%.2g\t%.2g\t \t%.2g\t \t%.2g\t \t%.2g\t \t%.2g" % (elapsedTime, i, vMean, vStd, iMean, iStd))
        i += ch["step_size"]


def ivscan(hw, args):

    with open(args.json) as f:
        config = json.load(f)

    if args.what == "iv" or args.what == "both":
        doSensorIV(config)

    if args.what == "vi" or args.what == "both":
        doModuleVI(config)


def toInfluxDB(points, host='localhost', port=8086):
    dbname = 'dcsDB' # this is just a temporal thing

    client = InfluxDBClient(host='127.0.0.1', port=8086, database=dbname)
    client.write_points(points, time_precision = 'u')


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-n",
        "--name",
        default="PS",
        help="Name of the power supply from equipment list",
    )
    parser.add_argument(
        "-c",
        "--channel",
        default="",
        help="Set PS channel (name or number when used in conjunction with --name) to control",
    )
    parser.add_argument(
        "-w",
        "--what",
        required=True,
        help="'iv' for IV curve, 'vi' for vi curve. 'both' for both)",
    )
    parser.add_argument(
        "-j",
        "--json",
        required=True,
        help="path to ivscan.json",
    )
    parser.add_argument(
        "-e",
        "--equip",
        required=True,
        help="powersupply.json Configuration file with the power supply definition",
    )
    parser.add_argument(
        "-d",
        "--debug",
        help="Enable more verbose printout, use multiple for increased debug level",
        action="store_true",
    )

    subparsers = parser.add_subparsers(help="command action")
    parser_set_current = subparsers.add_parser(
        "set-current", help="Set current [A] with maximum voltage [V]"
    )
    parser_set_current.add_argument("current", type=float)
    parser_set_current.add_argument(
        "max_voltage",
        type=float,
        nargs="?",
        default=-1,
        help="Set maximum voltage, -1 for no protection.",
    )
    parser_meas_voltage = subparsers.add_parser(
        "meas-voltage", help="Get reading of voltage [V]"
    )
    parser_meas_voltage.set_defaults(func=meas_voltage)

    parser_power_on = subparsers.add_parser(
        "power-on",
        help="Power ON PS, optionally setting voltage to V in Volts and current to I in Ampere",
    )
    parser_power_on.add_argument("voltage_level", nargs="?", type=float)
    parser_power_on.add_argument("current_protect", nargs="?", type=float)
    parser_power_on.set_defaults(func=power_on)

    parser_power_off = subparsers.add_parser("power-off", help="Power OFF PS")
    parser_power_off.set_defaults(func=power_off)


    args = parser.parse_args()



    # - - - - - - - - - -

    hw = labRemote.ec.EquipConf()
    hw.setHardwareConfig(args.equip)

    ivscan(hw, args)





