#!/bin/bash
#################################
# Contacts: Eunchong Kim, Arisa Kubota
# Email: eunchong.kim at cern.ch, arisa.kubota at cern.ch
# Date: April 2019
# Project: Local Database for Yarr
# Description: Create config file 
# Usage: ./createConfig.sh [-a <rd53a/fei4b>*] [-m <SerialNumber>*] [-c <ChipNum>] [-r <ControllerCfg>] [-d] [-R]
################################
### setting YARR ###############                                                                                


cd $YARR_DIR/configs
mkdir -p chipConfigs
################################         

# Change fixed tx channel and rx start channel
tx_fix=0
rx_start=0

# default parameters
reset=false
account=false
now=`date +"%y%m%d%H%M"`

# Usage
function usage {
    cat <<EOF

Usage:
    ./$(basename ${0}) [-a <rd53a/fei4b>*] [-m SN*] [-c Chips] [-r Controller] [-d] [-R]

Options:
    -a <rd53a/fei4b>  asic type (*req.)
    -m <str>          serial number (*req.)
    -c <int>          number of chips (req. in first creation)
    -r <path>         controller config file   default: ./controller/specCfg.json
    -d                upload into databse
    -R                reset all config files

EOF
}

while getopts a:m:c:r:dRI: OPT
do
    case ${OPT} in
        a ) asic=${OPTARG} ;;
        m ) sn=${OPTARG} ;;
        c ) chips=${OPTARG} ;;
        r ) controller=${OPTARG} ;;
        d ) account=true ;;
        R ) reset=true ;;
        * ) usage
            exit ;;
    esac
done

# asic type
if [ -z ${asic} ]; then
    echo "Please give \"rd53a\" or \"fei4b\" with '-a'."
    usage
    exit
elif [ ${asic} == "rd53a" ]; then
    chiptype="RD53A"
    chipid="ChipId"
    name="Name"
    john="JohnDoe_0"
elif [ ${asic} == "fei4b" ]; then
    chiptype="FEI4B"
    chipid="chipId"
    name="name"
    john="JohnDoe"
else
    echo "Please give \"rd53a\" or \"fei4b\" with '-a'."
    usage
    exit
fi

# serial number
if [ -z ${sn} ]; then
    echo "Please give serial number with '-m'."
    usage
    exit
fi

# Make directory for module data in ScanOperator

mkdir -p $SOBASE/configs/${asic}/${sn}
echo -e "Created dir: $SOBASE/configs/${asic}/${sn}"

# Make directory for config file
#if [ -f $YARR_DIR/configs/connectivity/connectivity.json ]; then

# chips
if [ -z ${chips} ]; then
    echo "Please give the number of chips with '-c'."
    usage
    exit
fi
echo ${chips} | grep [^0-9] > /dev/null 2>&1
if [ $? -eq 0 ]; then
    echo "Please give an integral as number of chips with '-c'. "
    usage
    exit
fi

# controller config
if [ -z ${controller} ]; then
    controller=controller/specCfg.json
fi
if [ ! -f ${controller} ]; then
    echo "Not exist controller \"${controller}\"."
    usage
    exit
fi

cnt=0
chipIds=()
chipNames=()
while [ ${cnt} -lt ${chips} ]; do
    chipName="${sn}_chip${cnt}"
    echo "Create ${YARR_DIR}/configs/chipConfigs/${chipName}.json"
    chipNames+=( ${chipName} )
    #echo "Set chipId for chip${cnt}... [#(chipId)] "
    #read answer
    #unset id
    id=$(jq -j '.modules.'"${sn}.chips[$cnt].id" $CFGFILE)
    cnt=$(( cnt + 1 ))
    chipIds+=( ${id} )
done
echo "---------------------"

# Confirmation
cnt=0
echo " "
echo "serial number:     ${sn}"
echo "controller config: ${controller}"
echo "number of chips:   ${chips}"
while [ ${cnt} -lt ${chips} ]; do
    cnt=$(( cnt + 1 ))
    echo "** ${sn}_chip${cnt}"
    echo "    chip serial number: ${chipNames[$((cnt-1))]}"
    echo "    chipId: ${chipIds[$((cnt-1))]}"
done
echo " "
read -n 1 -s -r -p "Press any key to continue"
echo ""

# Make controller for this module
if [ ${controller} != $SOBASE/configs/${asic}/${sn}/controller.json ]; then
    echo "Created ${YARR_DIR}/configs/controller/${sn}_controller.json"
    #cp ${controller} ${YARR_DIR}/configs/controller/${sn}_controller.json
    #ln -s ${YARR_DIR}/configs/controller/${sn}_controller.json $SOBASE/configs/${asic}/${sn}/controller.json
    cp ${controller} $SOBASE/configs/${asic}/${sn}/controller.json
fi

# Make connectivity.json
echo "Created $SOBASE/configs/$asic/$sn/connectivity.json"
echo "{" > $SOBASE/configs/$asic/$sn/connectivity.json
echo "    \"stage\": \"Testing\"," >> $SOBASE/configs/$asic/$sn/connectivity.json
echo "    \"module\": {" >> $SOBASE/configs/$asic/$sn/connectivity.json
echo "        \"serialNumber\": \"${sn}\"," >> $SOBASE/configs/$asic/$sn/connectivity.json
echo "        \"componentType\": \"Module\"" >> $SOBASE/configs/$asic/$sn/connectivity.json
echo "    }," >> $SOBASE/configs/$asic/$sn/connectivity.json
echo "    \"chipType\" : \"${chiptype}\"," >> $SOBASE/configs/$asic/$sn/connectivity.json

cnt=0
echo "    \"chips\" : [" >> $SOBASE/configs/$asic/$sn/connectivity.json
while [ ${cnt} -lt ${chips} ]; do
    cnt=$(( cnt + 1 ))
    echo "Created ${sn}_chip${cnt}.json"

    cp defaults/default_${asic}.json $SOBASE/configs/${asic}/${sn}/${sn}_chip${cnt}.json
    sed -i "/${chipid}/s/0/${chipIds[$((cnt-1))]}/g" $SOBASE/configs/${asic}/${sn}/${sn}_chip${cnt}.json
    sed -i "/${name}/s/JohnDoe_0/${chipNames[$((cnt-1))]}/g" $SOBASE/configs/${asic}/${sn}/${sn}_chip${cnt}.json

    rx_ch=$(( cnt - 1 + rx_start ))
    echo "        {" >> $SOBASE/configs/$asic/$sn/connectivity.json
    echo "            \"serialNumber\": \"${chipNames[$((cnt-1))]}\"," >> $SOBASE/configs/$asic/$sn/connectivity.json
    echo "            \"componentType\": \"Front-end Chip\"," >> $SOBASE/configs/$asic/$sn/connectivity.json
    echo "            \"chipId\": ${chipIds[$((cnt-1))]}," >> $SOBASE/configs/$asic/$sn/connectivity.json
    echo "            \"config\" : \"$SOBASE/configs/${asic}/${sn}/${sn}_chip${cnt}.json\"," >> $SOBASE/configs/$asic/$sn/connectivity.json
    echo "            \"tx\" : ${tx_fix}," >> $SOBASE/configs/$asic/$sn/connectivity.json
    echo "            \"rx\" : ${rx_ch}" >> $SOBASE/configs/$asic/$sn/connectivity.json
    
    if [ ${cnt} -ne ${chips} ]; then
        echo "        }," >> $SOBASE/configs/$asic/$sn/connectivity.json
    else
        echo "        }" >> $SOBASE/configs/$asic/$sn/connectivity.json
    fi
done
echo "    ]" >> $SOBASE/configs/$asic/$sn/connectivity.json
echo "}" >> $SOBASE/configs/$asic/$sn/connectivity.json


cd ../

# Register module and chips component to DB
if [ ! -z ${account} ]; then
    echo "./bin/dbAccessor -C -c $SOBASE/configs/$asic/$sn/connectivity.json"
    ./bin/dbAccessor -C -c $SOBASE/configs/$asic/$sn/connectivity.json 
fi
